/*  www.sgm.siedlce.pl, program do wykorzystania tylko na wlasny uzytek

SGM SlowServo:
realizacja funkcji "slow servo" dla servo2 i pary serwo1, servo3
od polozenia 'servo1min' do polozenia 'servo1max'(lub z rewersem)

serwo1 i serwo3 dzialaja jednoczesnie (ale z rewersem
w stosunku do siebie), uruchamiane przy zmianie pozycji
switch'a z min --> 1/2  (Low --> Mid)

serwo2 uruchamiane przy zmianie pozycji switch'a z 1/2 --> max
(Mid -->Max)

zworka JP blokuje po wlaczeniu zasilania dekodowanie sygnalu RC

pomiar kanalu RC na wejsciu (D2)
w oparciu o przerwanie; obsługa przerwania --> procedury:
void pulse_rising()
void pulse_falling()

dozwolony zakres dlugosci impulsu: 900-2100uS

serwa podlaczone do (D10, D11, D12))

opcjonalnie wydruk zmierzonych wartosci na port szeregowy

pomiar czasu przy uzyciu biblioteki standardowej funkcji micros()

usrednianie zmierzonego sygnału (4 ostatnie probki)

status przelacznika RC na diodzie LED D13,
powolne miganie - SW=Low
szybkie miganie - SW=Mid
świecenie       - SW=High

*/

//###################################################################################

//parametry uzytkownika dla serw 1,3 (para, serwo3 pracuje odwrotnie niz serwo1)
const byte servo1rev = 1;    //rewers pary 1,3  (0=bez_reversu, 1=revers)
const int servo1min = 1020;  //minimalne polozenie serwa (microsec)
const int servo1max = 2000;  //maksymalne polozenie serwa (microsec)
const int servo1step = 5;    //zmiana (krok) wartosci wychylenia (microsec)
const int tGapS1 = 20;       //odstep czasowy (krok) zmiany polozenia serwa (milisec)

//parametry uzytkownika dla serwa 2
const byte servo2rev = 1;    //rewers serwa 2 (0=bez_reversu, 1=revers)
const int servo2min = 1050;  //minimalne polozenie serwa (microsec)
const int servo2max = 1950;  //maksymalne polozenie serwa (microsec)
const int servo2step = 5;    //zmiana (krok) wartosci wychylenia (microsec)
const int tGapS2 = 20;       //odstep czasowy (krok) zmiany polozenia serwa (milisec)

//####################################################################################


//deklaracje zmiennych

volatile unsigned long pulse1_start = 0;   //zmienne dla micros()
unsigned long pulse1_micros = 0;           //zmienna na dlugosc impulsu
                                           //kanalowego PPM wg micros()

int pulse_uS;                //zmienne pomocnicze
int servoTmp;
int buf1;                    //zmienne bufora impulsow
int buf2;
int buf3;
byte SW_pos;                 //zmienna na polozenie przelacznika RC
const int SW_low = 1300;     //stala - SW w polozeniu dolnym
const int SW_high = 1700;    //stala - SW w polozeniu gornym
const int LED = 13;          // LED na Pin 13
byte LED_status = 0;         //status diody LED

#include <Servo.h>           // dolaczenie biblioteki 'servo'
Servo myservo1;              // utworzenie obiektu do sterowania serwem 
Servo myservo2;
Servo myservo3;

const int servoFS = 1000;    //wartosc domyslna impulsu RC
int servoP;                  //zmienna na sygnal wyjsciowy servo

const int tGap = 500;        //opcjonalnie - co ile wydruk danych (msec)
unsigned long t_start;       //zmienne do odliczania czasu przy wydruku
unsigned long t_count; 

const int tGapLL = 500;      //stale i zmienne do obslugi migania diody
const int tGapLH = 200;
unsigned long t_startL;
unsigned long t_countL;  

//dodatkowe zmienne dla serw 1,3 
int servo1pos;               //zmienna na polozenie serwa 1
byte servo1mode;             //status wlaczenia
int servo3pos;               //zmienna na polozenie serwa 3
unsigned long t_startS1;
unsigned long t_countS1;

//dodatkowe zmienne dla serwa 2
int servo2pos;               //zmienna na polozenie serwa 2
byte servo2mode;             //status wlaczenia
unsigned long t_startS2;
unsigned long t_countS2;

//zmienne dla zworki zabezpieczajacej JP - sprzetowa blokada ruchu serw

int ARead;
byte JPout = 0;  //0 zworka zalozona, 1 zdjeta
int JPservo1;    //domyslne wartosci serw przy zalozonej zworce
int JPservo2;
int JPservo3;


//inicjalizacja programu
void setup() { 

  // inicjalizacja zmiennych pomiaru
  SW_pos = 0;
  servoP = servoFS;
  buf1 = servoFS;
  buf2 = buf1;
  buf3 = buf2;
  
  // inicjalizacja serwa 1 do D8
  servo1mode = 0;               //serwo1 na minimum
  if (servo1rev == 1) {
   servo1pos = servo1max;
   servo3pos = servo1min;
  } else {  
    servo1pos = servo1min;
    servo3pos = servo1max;
  }  
  JPservo1 = servo1pos;  //domyslne wartosci zaleznie od rewersu a nie bezwzglednie
  JPservo3 = servo3pos;
  
  // 10,11,12
  myservo1.attach(11);      
  myservo1.writeMicroseconds(servo1pos);
  myservo3.attach(10);      
  myservo3.writeMicroseconds(servo3pos);
  
  // inicjalizacja serwa 2
  servo2mode = 0;               //serwo2 na minimum
  if (servo2rev == 1) {
   servo2pos = servo2max;
  } else {  
   servo2pos = servo2min;
  }
  JPservo2 = servo2pos;
  myservo2.attach(12);      
  myservo2.writeMicroseconds(servo2pos);
  
  //inicjalizacja portu szeregowego (opcjonalny wydruk danych)
  //Serial.begin(115200); 
  
  //inicjalizacja przerwania 0 (D2, Arduino UNO)
  attachInterrupt(0, pulse_rising, RISING);
  t_start = millis();  
}

//glowna petla programu
void loop() {
  
  //opcjonalnie obsulga drukowania co 'tGap' pomiaru na port szeregowy  
  
  t_count = millis(); 
  //jesli uplynelo tGap to drukuj
  if ((t_count - t_start) >= tGap) 
  {
    t_start = t_count;
    if (JPout == 0) {
     ARead = analogRead(5);
     if (ARead > 300) {
      JPout = 1; 
     }  
    }
    /*
    // wydruk parametrow
    Serial.print(servoP);
    Serial.print(",  ");
    Serial.print(SW_pos);
    Serial.print(",  ");
    Serial.print(servo1mode);
    Serial.print(",  ");
    Serial.print(servo1pos);
    Serial.print(",     ");
    Serial.print(servo3pos);
    Serial.print(",     ");
    
    Serial.print(servo2mode);
    Serial.print(",  ");
    Serial.println(servo2pos);
    //
    */
  } // if (t_count - t_start), obsluga drukowania
  
  
  if (JPout == 1) {     //jesli zworka zdjeta, to serwa wg SW
  
  
   switch (SW_pos) {  
    
    case 0:          // polozenie LOW, dioda miga powoli
     t_countL = millis();
     if ((t_countL - t_startL) >= tGapLL) {
      t_startL = t_countL; 
      if (LED_status == 0) {
       digitalWrite(LED, HIGH);
       LED_status = 1; 
      } else {
       digitalWrite(LED, LOW);
       LED_status = 0; 
      } 
     } //if ((t_countL      
    break; // case 0
   
    case 1:          // polozenie MID, dioda miga szybko
     t_countL = millis();
     if ((t_countL - t_startL) >= tGapLH) {
      t_startL = t_countL; 
      if (LED_status == 0) {
       digitalWrite(LED, HIGH);
       LED_status = 1; 
      } else {
       digitalWrite(LED, LOW);
       LED_status = 0; 
      } 
     } //if ((t_countL 
    break; // case 1
    
    case 2:          // polozenie HIGH, dioda swieci ciagle
     digitalWrite(LED, HIGH);
     LED_status = 1;     
    break; // case 2    
   
   } //switch (SW_pos)
  
   
   //zaleznosci (SW # serwo1)
   // SW_pos  LOW --> MID
   if ((servo1mode == 0) && (SW_pos >= 1)){ //wlaczenie serwa 1
     servo1mode = 1;
   } 
   // SW_pos  MID--> LOW
   if ((servo1mode == 2) && (SW_pos == 0)){ //wlaczenie serwa 1
     servo1mode = 3;
   } 
 
   
   //zaleznosci (SW # serwo2)
   // SW_pos  MID--> HIGH
   if ((servo2mode == 0) && (SW_pos == 2)){ //wlaczenie serwa 2
     servo2mode = 1;
   } 
   // SW_pos  HIGH --> MID/LOW
   if ((servo2mode == 2) && (SW_pos <= 1)){ //wlaczenie serwa 2
     servo2mode = 3;
   } 
 

 
 // obsluga ruchu serw
 
   if (servo1rev == 1) { //obsluga servo 1,3 w zaleznosci czy rewers
    
    //serwo 1,3 z rewersem 
    switch (servo1mode) {  
      
     case 0:          // polozenie LOW, serwo w minimum, SW LOW
     break; // case 0
    
     case 1:          // polozenie mid, ruch od min do max
      if (servo1pos > servo1min) {
       t_countS1 = millis();
       if ((t_countS1 - t_startS1) >= tGapS1) {
        t_startS1 = t_countS1;
        //servo1pos = servo1pos + servo1step;
        //servo3pos = servo3pos - servo1step;
        servo1pos = servo1pos - servo1step;
        servo3pos = servo3pos + servo1step;
       } 
      } else {
       servo1pos = servo1min;
       servo3pos = servo1max;
       servo1mode = 2;   
      } //if servo1pos < servo1max 
     break; // case 1
    
     case 3:          // polozenie mid, ruch od min do max
      if (servo1pos < servo1max) {
       t_countS1 = millis();
       if ((t_countS1 - t_startS1) >= tGapS1) {
        t_startS1 = t_countS1;
        servo1pos = servo1pos + servo1step;
        servo3pos = servo3pos - servo1step;     
       } 
      } else {
       servo1pos = servo1max;
       servo3pos = servo1min;
       servo1mode = 0;
      } //if servo1pos > servo1max 
     break; // case 1  
   
    } //switch (servo1mode)
   
   } else {
    //serwo 1,3 bez rewersu
    switch (servo1mode) {  
      
     case 0:          // polozenie LOW, serwo w minimum, SW LOW
     break; // case 0
    
     case 1:          // polozenie mid, ruch od min do max
      if (servo1pos < servo1max) {
       t_countS1 = millis();
       if ((t_countS1 - t_startS1) >= tGapS1) {
        t_startS1 = t_countS1;
        servo1pos = servo1pos + servo1step;
        servo3pos = servo3pos - servo1step;
       } 
      } else {
       servo1pos = servo1max;
       servo3pos = servo1min;
       servo1mode = 2;   
      } //if servo1pos < servo1max 
     break; // case 1
    
     case 3:          // polozenie mid, ruch od min do max
      if (servo1pos > servo1min) {
       t_countS1 = millis();
       if ((t_countS1 - t_startS1) >= tGapS1) {
        t_startS1 = t_countS1;
        servo1pos = servo1pos - servo1step;
        servo3pos = servo3pos + servo1step;
       } 
      } else {
       servo1pos = servo1min;
       servo3pos = servo1max;
       servo1mode = 0;
      } //if servo1pos > servo1max 
     break; // case 1  
   
    } //switch (servo1mode)  
   }
  
   if (servo2rev == 1) { //obsluga servo 2 w zaleznosci czy rewers

     //serwo 2 z rewersem
     switch (servo2mode) {  
      
     case 0:          // polozenie LOW/MID, serwo w minimum, 
     break; // case 0
    
     case 1:          // polozenie mid, ruch od min do max
      if (servo2pos > servo2min) {
       t_countS2 = millis();
       if ((t_countS2 - t_startS2) >= tGapS2) {
        t_startS2 = t_countS2;
        servo2pos = servo2pos - servo2step;
       } 
      } else {
       servo2pos = servo2min;
       servo2mode = 2;   
      } //if servo2pos < servo1max 
     break; // case 1
    
     case 3:          // polozenie HIGH, ruch od min do max
      if (servo2pos < servo2max) {
       t_countS2 = millis();
       if ((t_countS2 - t_startS2) >= tGapS2) {
        t_startS2 = t_countS2;
        servo2pos = servo2pos + servo2step;
       } 
      } else {
       servo2pos = servo2max;
       servo2mode = 0;
      } //if servo2pos < servo2max 
     break; // case 1  
   
    } //switch (servo2mode)  
    
   } else {  
   
     //obsluga serwo 2 bez rewersu
     switch (servo2mode) {  
      
     case 0:          // polozenie LOW/MID, serwo w minimum, 
     break; // case 0
    
     case 1:          // polozenie mid, ruch od min do max
      if (servo2pos < servo2max) {
       t_countS2 = millis();
       if ((t_countS2 - t_startS2) >= tGapS2) {
        t_startS2 = t_countS2;
        servo2pos = servo2pos + servo2step;
       } 
      } else {
       servo2pos = servo2max;
       servo2mode = 2;   
      } //if servo2pos < servo2max 
     break; // case 1
    
     case 3:          // polozenie HIGH, ruch od min do max
      if (servo2pos > servo2min) {
       t_countS2 = millis();
       if ((t_countS2 - t_startS2) >= tGapS2) {
        t_startS2 = t_countS2;
        servo2pos = servo2pos - servo2step;
       } 
      } else {
       servo2pos = servo2min;
       servo2mode = 0;
      } //if servo2pos > servo2min 
     break; // case 1  
   
    } //switch (servo2mode) (bez rewersu) 
  
   } //if (servo2rev == 1)
   
   myservo1.writeMicroseconds(servo1pos);
   myservo3.writeMicroseconds(servo3pos);
   myservo2.writeMicroseconds(servo2pos);
 
  } else { //zworka zalozona, na serwa default
  
   myservo1.writeMicroseconds(JPservo1);
   myservo3.writeMicroseconds(JPservo3);
   myservo2.writeMicroseconds(JPservo2);
  
  } //if (JPout == 1) 
  
} // koniec glownej petli



// obsluga przerwania (rising), poczatek impulsu
void pulse_rising() {
  pulse1_start = micros();
  attachInterrupt(0, pulse_falling, FALLING);  
}


// obsluga przerwania (falling), koniec impulsu 
void pulse_falling() {
  //dlugosc impulsu wg micros()
  pulse1_micros = micros()-pulse1_start;
  pulse_uS = pulse1_micros;
  //sprawdzenie czy dlugosc impulsu z dozwolonego zakresu
  if ((pulse_uS > 900) && (pulse_uS < 2100)) {
     servoTmp = pulse_uS + buf3 + buf2 + buf1; 
     servoP = servoTmp / 4;
     buf3 = buf2;
     buf2 = buf1;
     buf1 = servoP;
     
     if ((servoP > SW_low) && (servoP < SW_high)) {
      SW_pos = 1;  
     }
     if (servoP <= SW_low)  {
      SW_pos = 0;  
     }
     if (servoP >= SW_high) {
      SW_pos = 2;  
     } 
     //myservo.writeMicroseconds(servoP);
  }//if ((pulse_uS > 900
 
  attachInterrupt(0, pulse_rising, RISING);        
}





